from sqlalchemy import String, Column, DateTime, Float, Integer, ForeignKey
from sqlalchemy.orm import relationship
from .sap_model import SapBonanzaBase as SBB

class OrdemTrabalho(SBB):
    __tablename__ = 'sap_ordem_trabalho'

    Code = Column('Code', String, primary_key=True)
    Name = Column('Name', String)
    DocEntry = Column('DocEntry', Integer)
    Canceled = Column('Canceled', String)
    Object = Column('Object', String)
    LogInst = Column('LogInst', String)
    UserSign = Column('UserSign', Integer)
    Transfered = Column('Transfered', String)
    CreateDate = Column('CreateDate', DateTime)
    CreateTime = Column('CreateTime', String)
    UpdateDate = Column('UpdateDate', DateTime)
    UpdateTime = Column('UpdateTime', String)
    DataSource = Column('DataSource', String)
    U_NoOT = Column('U_NoOT', String)
    U_NoUni = Column('U_NoUni', String)
    U_NoCon = Column('U_NoCon', String)
    U_Plac = Column('U_Plac', String)
    U_Marc = Column('U_Marc', String)
    U_Esti = Column('U_Esti', String)
    U_NoVis = Column('U_NoVis', String)
    U_EstVis = Column('U_EstVis', String)
    U_VIN = Column('U_VIN', String)
    U_TipOT = Column('U_TipOT', String)
    U_EstW = Column('U_EstW', String)
    U_FCom = Column('U_FCom', String)
    U_FApe = Column('U_FApe', DateTime)
    U_FFin = Column('U_FFin', DateTime)
    U_EstO = Column('U_EstO', String)
    U_Ase = Column('U_Ase', String)
    U_EncO = Column('U_EncO', String)
    U_Obse = Column('U_Obse', String)
    U_CodEst = Column('U_CodEst', String)
    U_CodMar = Column('U_CodMar', String)
    U_Cotiz = Column('U_Cotiz', String)
    U_RCot = Column('U_RCot', String)
    U_DocEntry = Column('U_DocEntry', Integer)
    U_OTRef = Column('U_OTRef', String)
    U_NGas = Column('U_NGas', String)
    U_Sucu = Column('U_Sucu', String)
    U_Mode = Column('U_Mode', String)
    U_CEst = Column('U_CEst', String)
    U_CMod = Column('U_CMod', String)
    U_CMar = Column('U_CMar', String)
    U_Ano = Column('U_Ano', String)
    U_CodCli = Column('U_CodCli', String)
    U_NCli = Column('U_NCli', String)
    U_CodCOT = Column('U_CodCOT', String)
    U_NCliOT = Column('U_NCliOT', String)
    U_Cor = Column('U_Cor', String)
    U_Tel = Column('U_Tel', String)
    U_MOReal = Column('U_MOReal', Float)
    U_MOEsta = Column('U_MOEsta', Float)
    U_NoCita = Column('U_NoCita', String)
    U_FecVta = Column('U_FecVta', DateTime)
    U_Color = Column('U_Color', String)
    U_FechPro = Column('U_FechPro', DateTime)
    U_Repro = Column('U_Repro', String)
    U_Esp_Re = Column('U_Esp_Re', String)
    U_DEstO = Column('U_DEstO', String)
    U_km = Column('U_km', Float)
    U_HCom = Column('U_HCom', String)
    U_HApe = Column('U_HApe', String)
    U_HFin = Column('U_HFin', String)
    U_FCerr = Column('U_FCerr', DateTime)
    U_FFact = Column('U_FFact', DateTime)
    U_FEntr = Column('U_FEntr', DateTime)
    U_FRec = Column('U_FRec', DateTime)
    U_HRec = Column('U_HRec', String)
    U_HMot = Column('U_HMot', Float)
    U_SCGD_ObservInterna = Column('U_SCGD_ObservInterna', String)
    U_ObservCierre = Column('U_ObservCierre', String)
    U_ObservDiag = Column('U_ObservDiag', String)
    U_CreadoPor = Column('U_CreadoPor', String)
    U_IdEstOTTC = Column('U_IdEstOTTC', Float)
    U_Attachment = Column('U_Attachment', String)
    U_Bahia = Column('U_Bahia', String)
    U_FechaCierre = Column('U_FechaCierre', DateTime)
    U_HoraCierre = Column('U_HoraCierre', String)
    U_NUrea = Column('U_NUrea', String)
    U_ComentariosG = Column('U_ComentariosG', String)

    # Relacionamento com OrdemTrabalhoCollection
    ordem_trabalho_collections = relationship("OrdemTrabalhoCollection", back_populates="ordem_trabalho")

    def __repr__(self):
        return f"<OrdemTrabalho(code='{self.Code}', description='{self.U_Marc} - {self.U_Mode}')>"

class OrdemTrabalhoCollection(SBB):
    __tablename__ = 'sap_ordem_trabalho_collection'

    id = Column(Integer, primary_key=True, autoincrement=True)
    Code = Column('Code', String, ForeignKey(f'{SBB.__table_args__.get("schema")}.sap_ordem_trabalho.Code'))
    LineId = Column('LineId', Integer)
    Object = Column('Object', String)
    LogInst = Column('LogInst', String)
    U_Colab = Column('U_Colab', String)
    U_TMin = Column('U_TMin', Float)
    U_RePro = Column('U_RePro', String)
    U_NoFas = Column('U_NoFas', String)
    U_Estad = Column('U_Estad', String)
    U_IdAct = Column('U_IdAct', String)
    U_CosRe = Column('U_CosRe', Float)
    U_CosEst = Column('U_CosEst', Float)
    U_ReAsig = Column('U_ReAsig', String)
    U_FIni = Column('U_FIni', String)
    U_FFin = Column('U_FFin', String)
    U_HoraIni = Column('U_HoraIni', String)
    U_FechPro = Column('U_FechPro', DateTime)
    U_CodFas = Column('U_CodFas', String)
    U_DFIni = Column('U_DFIni', DateTime)
    U_HFIni = Column('U_HFIni', String)
    U_DFFin = Column('U_DFFin', DateTime)
    U_HFFin = Column('U_HFFin', String)
    U_SuspensionHorario = Column('U_SuspensionHorario', String)
    U_FchaAsignacion = Column('U_FchaAsignacion', DateTime)
    U_HoraAsignacion = Column('U_HoraAsignacion', String)
    U_TiempoExt = Column('U_TiempoExt', String)
    U_blqServicio = Column('U_blqServicio', String)
    U_CtrlBloque = Column('U_CtrlBloque', String)
    U_SCGD_TCCOmms = Column('U_SCGD_TCCOmms', String)
    U_TAdicional = Column('U_TAdicional', String)
    U_TMinBloqueo = Column('U_TMinBloqueo', Float)
    U_TMinBloqParcial = Column('U_TMinBloqParcial', Float)

    # Relacionamento com OrdemTrabalho
    ordem_trabalho = relationship("OrdemTrabalho", back_populates="ordem_trabalho_collections")

    def __repr__(self):
        return f"<OrdemTrabalhoCollection(line_id='{self.LineId}', colaborador='{self.U_Colab}')>"
